const puppeteer = require('puppeteer-extra');
const StealthPlugin = require('puppeteer-extra-plugin-stealth');
const AdBlockerPlugin = require('puppeteer-extra-plugin-adblocker');
const ArgsParser = require('./classes/utils/ArgsParser');
const MediaLinksFile = require('./classes/filesystem/MediaLinksFile');
const KissAsianScraper = require('./classes/scraping/KissAsianScraper');
const JDownloaderClient = require('./classes/download/JDownloaderClient');
const Utils = require('./classes/utils/Utils');
const credentials = require('../data/credentials.json');

const args = new ArgsParser().parse(process.argv.slice(2));

puppeteer.use(StealthPlugin());
puppeteer.use(AdBlockerPlugin({ blockTrackers: true }));

const singleMediaProcedure = async(scraper, downloader, mediaURL) => {
  const media = await scraper.getMediaData(mediaURL);
  await scraper.populateMediaDownloadURLs(media);

  await downloader.addLinks(media);

  const crawledLinks = await downloader.getCrawledLinks(media.downloadURLs);
  const renamedCrawledLinks = downloader.getRenamedCrawledLinks(crawledLinks, media);
  await downloader.renameCrawledLinks(renamedCrawledLinks);
  
  await downloader.startDownload(crawledLinks);
};

const main = async() => {
  const browser = await puppeteer.launch({ headless: !args.display, slowMo: 250 });
  const page = await browser.newPage();

  const file = new MediaLinksFile(args.directory);
  const mediaToDownload = await file.read();

  const scraper = new KissAsianScraper(page);
  const downloader = new JDownloaderClient(credentials.jdownloader);

  await downloader.load();
  await scraper.load();
  await scraper.authenticate(credentials.kissasian);

  await Utils.mapSeries(mediaToDownload, async(url) => {
    await singleMediaProcedure(scraper, downloader, url);
    await file.removeFirstLink();
  });

  await file.remove();
  await downloader.finish();
  await scraper.finish(browser);
  process.exit(0);
};

main();
